/*
 * Copyright (c) 2003-2007 Erez Zadok
 * Copyright (c) 2003-2007 Stony Brook University
 * Copyright (c) 2003-2007 The Research Foundation of State University of New York
 * Copyright (c) 2004-2007 Nikolai Joukov
 * Copyright (c) 2003-2005 Akshat Aranya
 * Copyright (c) 2006-2007 Gopala Surya
 *
 * For specific licensing information, see the COPYING file distributed with
 * this package, or get one from ftp://ftp.filesystems.org/pub/fist/COPYING.
 *
 * This Copyright notice must be kept intact and distributed with all
 * fistgen sources INCLUDING sources generated by fistgen.
 */
/*
 * This file is the assembly driver wrapper for tracing
 * This contains one function each for every file system operation
 */

#include "tracer.h"

#define dest_for_each(i, dest, sb) \
    for (i = 0; i < MAX_DESTINATIONS && (dest = (sb)->dest + i); i++)

/*
#define dest_for_each(i, dest, sb) \
    for (i = 0; i < 0 && (dest = (sb)->dest + i); i++)
*/
#define MAKE_CALL1(sb, op, fname, inode, dentry, arg0, arg1) \
	ASSERT(sb); \
	do { \
	    struct trace_destination *dest; \
	    int i; \
	    dest_for_each(i, dest, (sb)) \
            if (evaluate(dest, (op), (inode), (dentry)) && dest->td_ad->ad_ops->fname) \
	        (dest)->td_ad->ad_ops->fname((dest)->td_ad, (dest), (arg0), (arg1)); \
	} while (0)


#define MAKE_CALL2(sb, op, fname, inode, dentry, arg0, arg1, arg2) \
	ASSERT(sb); \
	do { \
	    struct trace_destination *dest; \
	    int i; \
	    dest_for_each(i, dest, (sb)) \
            if (evaluate(dest, (op), (inode), (dentry)) && dest->td_ad->ad_ops->fname) \
	        (dest)->td_ad->ad_ops->fname((dest)->td_ad, (dest), (arg0), (arg1), (arg2)); \
	} while (0)

#define MAKE_CALL3(sb, op, fname, inode, dentry, arg0, arg1, arg2, arg3) \
	ASSERT(sb); \
	do { \
	    struct trace_destination *dest; \
	    int i; \
	    dest_for_each(i, dest, (sb)) \
            if (evaluate(dest, (op), (inode), (dentry)) && dest->td_ad->ad_ops->fname) \
	        (dest)->td_ad->ad_ops->fname((dest)->td_ad, (dest), (arg0), (arg1), (arg2), (arg3)); \
	} while (0)

#define MAKE_CALL4(sb, op, fname, inode, dentry, arg0, arg1, arg2, arg3, arg4) \
	ASSERT(sb); \
	do { \
	    struct trace_destination *dest; \
	    int i; \
	    dest_for_each(i, dest, (sb)) \
            if (evaluate(dest, (op), (inode), (dentry)) && dest->td_ad->ad_ops->fname) \
	        (dest)->td_ad->ad_ops->fname((dest)->td_ad, (dest), (arg0), (arg1), (arg2), (arg3), (arg4)); \
	} while (0)


#define MAKE_CALL5(sb, op, fname, inode, dentry, arg0, arg1, arg2, arg3, arg4, arg5) \
	ASSERT(sb); \
	do { \
	    struct trace_destination *dest; \
	    int i; \
	    dest_for_each(i, dest, (sb)) \
            if (evaluate(dest, (op), (inode), (dentry)) && dest->td_ad->ad_ops->fname) \
	        (dest)->td_ad->ad_ops->fname((dest)->td_ad, (dest), (arg0), (arg1), (arg2), (arg3), (arg4), (arg5)); \
	} while (0)

void trace_llseek_pre(struct trace_sb *sb, struct file *file, loff_t offset, int origin, void **context)
{
	MAKE_CALL3(sb, OP_LLSEEK, ao_llseek_pre, file->f_dentry->d_inode, file->f_dentry, file, offset, origin, context);
}

void trace_llseek_post(struct trace_sb *sb, struct file *file, loff_t offset, int origin, loff_t ret, void **context)
{
	MAKE_CALL4(sb, OP_LLSEEK, ao_llseek_post, file->f_dentry->d_inode, file->f_dentry, file, offset, origin, ret, context);
}


void trace_open_pre(struct trace_sb *sb, struct inode *inode, struct file *file, void **context)
{
	MAKE_CALL2(sb, OP_OPEN, ao_open_pre, inode, file->f_dentry, inode, file, context);
}

void trace_open_post(struct trace_sb *sb, struct inode *inode, struct file *file, int ret, void **context)
{
	MAKE_CALL3(sb, OP_OPEN, ao_open_post, inode, file->f_dentry, inode, file, ret, context);
}

void trace_create_pre(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, int mode, void **context)
{
	MAKE_CALL3(sb, OP_CREATE, ao_create_pre, dir, dentry, dir, dentry, mode, context);
}

void trace_create_post(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, int mode, int ret, void **context)
{
	MAKE_CALL4(sb, OP_CREATE, ao_create_post, dir, dentry, dir, dentry, mode, ret, context);
}

void trace_lookup_pre(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, void **context)
{
	MAKE_CALL2(sb, OP_LOOKUP, ao_lookup_pre, dir, dentry, dir, dentry, context);
}

void trace_lookup_post(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, struct dentry *new_dentry, void **context)
{
	MAKE_CALL3(sb, OP_LOOKUP, ao_lookup_post, dir, dentry, dir, dentry, new_dentry, context);
}

void trace_link_pre(struct trace_sb *sb, struct dentry *old_dentry, struct inode *dir, struct dentry *new_dentry, void **context)
{
	MAKE_CALL3(sb, OP_LINK, ao_link_pre, dir, old_dentry, old_dentry, dir, new_dentry, context);
}

void trace_link_post(struct trace_sb *sb, struct dentry *old_dentry, struct inode *dir, struct dentry *new_dentry, int ret, void **context)
{
	MAKE_CALL4(sb, OP_LINK, ao_link_post, dir, old_dentry, old_dentry, dir, new_dentry, ret, context);
}

void trace_unlink_pre(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, void **context)
{
	MAKE_CALL2(sb, OP_UNLINK, ao_unlink_pre, dir, dentry, dir, dentry, context);
}

void trace_unlink_post(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, int ret, void **context)
{
	MAKE_CALL3(sb, OP_UNLINK, ao_unlink_post, dir, dentry, dir, dentry, ret, context);
}

void trace_symlink_pre(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, const char *symname, void **context)
{
	MAKE_CALL3(sb, OP_SYMLINK, ao_symlink_pre, dir, dentry, dir, dentry, symname, context);
}

void trace_symlink_post(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, const char *symname, int ret, void **context)
{
	MAKE_CALL4(sb, OP_SYMLINK, ao_symlink_post, dir, dentry, dir, dentry, symname, ret, context);
}

void trace_mkdir_pre(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, int mode, void **context)
{
	MAKE_CALL3(sb, OP_MKDIR, ao_mkdir_pre, dir, dentry, dir, dentry, mode, context);
}

void trace_mkdir_post(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, int mode, int ret, void **context)
{
	MAKE_CALL4(sb, OP_MKDIR, ao_mkdir_post, dir, dentry, dir, dentry, mode, ret, context);
}

void trace_rmdir_pre(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, void **context)
{
	MAKE_CALL2(sb, OP_RMDIR, ao_rmdir_pre, dir, dentry, dir, dentry, context);
}

void trace_rmdir_post(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, int ret, void **context)
{
	MAKE_CALL3(sb, OP_RMDIR, ao_rmdir_post, dir, dentry, dir, dentry, ret, context);
}

void trace_mknod_pre(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, int mode, int dev, void **context)
{
	MAKE_CALL4(sb, OP_MKNOD, ao_mknod_pre, dir, dentry, dir, dentry, mode, dev, context);
}

void trace_mknod_post(struct trace_sb *sb, struct inode *dir, struct dentry *dentry, int mode, int dev, int ret, void **context)
{
	MAKE_CALL5(sb, OP_MKNOD, ao_mknod_post, dir, dentry, dir, dentry, mode, dev, ret, context);
}
void trace_rename_pre(struct trace_sb *sb, struct inode *old_dir, struct dentry *old_dentry, struct inode *new_dir, struct dentry *new_dentry, void **context)
{
	MAKE_CALL4(sb, OP_RENAME, ao_rename_pre, old_dir, old_dentry, old_dir, old_dentry, new_dir, new_dentry, context);
}

void trace_rename_post(struct trace_sb *sb, struct inode *old_dir, struct dentry *old_dentry, struct inode *new_dir, struct dentry *new_dentry, int ret, void **context)
{
	MAKE_CALL5(sb, OP_RENAME, ao_rename_post, old_dir, old_dentry, old_dir, old_dentry, new_dir, new_dentry, ret, context);
}

void trace_readlink_pre(struct trace_sb *sb, struct dentry *dentry, char *buf, int bufsiz, void **context)
{
	MAKE_CALL3(sb, OP_READLINK, ao_readlink_pre, dentry->d_inode, dentry, dentry, buf, bufsiz, context);
}

void trace_readlink_post(struct trace_sb *sb, struct dentry *dentry, char *buf, int bufsiz, int ret, void **context)
{
	MAKE_CALL4(sb, OP_READLINK, ao_readlink_post, dentry->d_inode, dentry, dentry, buf, bufsiz, ret, context);
}

void trace_follow_link_pre(struct trace_sb *sb, struct dentry *dentry, struct nameidata *nd, void **context)
{
	MAKE_CALL2(sb, OP_FOLLOW_LINK, ao_follow_link_pre, dentry->d_inode, dentry, dentry, nd, context);
}

void trace_follow_link_post(struct trace_sb *sb, struct dentry *dentry, struct nameidata *nd, int ret, void **context)
{
	MAKE_CALL3(sb, OP_FOLLOW_LINK, ao_follow_link_post,  dentry->d_inode, dentry, dentry, nd, ret, context);
}

void trace_permission_pre(struct trace_sb *sb, inode_t *inode, int mask, void **context)
{
	MAKE_CALL2(sb, OP_PERMISSION, ao_permission_pre, inode, NULL, inode, mask, context);
}

void trace_permission_post(struct trace_sb *sb, inode_t *inode, int mask, int ret, void **context)
{
	MAKE_CALL3(sb, OP_PERMISSION, ao_permission_post, inode, NULL, inode,mask, ret, context);
}

void trace_inode_revalidate_pre(struct trace_sb *sb, struct dentry *dentry, void **context)
{
	MAKE_CALL1(sb, OP_INODE_REVALIDATE, ao_inode_revalidate_pre, dentry->d_inode, dentry, dentry, context);
}

void trace_inode_revalidate_post(struct trace_sb *sb, struct dentry *dentry, int ret, void **context)
{
	MAKE_CALL2(sb, OP_INODE_REVALIDATE, ao_inode_revalidate_post, dentry->d_inode, dentry, dentry, ret, context);
}

void trace_setattr_pre(struct trace_sb *sb, struct dentry *dentry, struct iattr *ia, void **context)
{
	MAKE_CALL2(sb, OP_SETATTR, ao_setattr_pre, dentry->d_inode, dentry, dentry, ia, context);
}

void trace_setattr_post(struct trace_sb *sb, struct dentry *dentry, struct iattr *ia, int ret, void **context)
{
	MAKE_CALL3(sb, OP_SETATTR, ao_setattr_post, dentry->d_inode, dentry, dentry, ia, ret, context);
}

void trace_read_pre(struct trace_sb *sb, file_t *file, char *buf, size_t count, loff_t *ppos, void **context)
{
	MAKE_CALL4(sb, OP_READ, ao_read_pre, file->f_dentry->d_inode, file->f_dentry, file, buf, count, ppos, context);
}

void trace_read_post(struct trace_sb *sb, file_t *file, char *buf, size_t count, loff_t *ppos, ssize_t ret, void **context)
{
	MAKE_CALL5(sb, OP_READ, ao_read_post, file->f_dentry->d_inode, file->f_dentry, file, buf, count, ppos, ret, context);
}

void trace_write_pre(struct trace_sb *sb, file_t *file, const char *buf, size_t count, loff_t *ppos, void **context)
{
	MAKE_CALL4(sb, OP_WRITE, ao_write_pre, file->f_dentry->d_inode, file->f_dentry, file, buf, count, ppos, context);
}

void trace_write_post(struct trace_sb *sb, file_t *file, const char *buf, size_t count, loff_t *ppos, ssize_t ret, void **context)
{
	MAKE_CALL5(sb, OP_WRITE, ao_write_post, file->f_dentry->d_inode, file->f_dentry, file, buf, count, ppos, ret, context);
}

void trace_readdir_pre(struct trace_sb *sb, file_t *file, void *dirent, filldir_t filldir, int count, void **context)
{
	MAKE_CALL4(sb, OP_READDIR, ao_readdir_pre, file->f_dentry->d_inode, file->f_dentry, file, dirent, filldir, count, context);
}
void trace_readdir_post(struct trace_sb *sb, file_t *file, void *dirent, filldir_t filldir, int count, int ret, void **context)
{
	MAKE_CALL5(sb, OP_READDIR, ao_readdir_post, file->f_dentry->d_inode, file->f_dentry, file, dirent, filldir, count, ret, context);
}


void trace_poll_pre(struct trace_sb *sb, file_t *file, poll_table *wait, void **context)
{
	MAKE_CALL2(sb, OP_POLL, ao_poll_pre, file->f_dentry->d_inode, file->f_dentry, file, wait, context);
}

void trace_poll_post(struct trace_sb *sb, file_t *file, poll_table *wait, int ret, void **context)
{
	MAKE_CALL3(sb, OP_POLL, ao_poll_post, file->f_dentry->d_inode, file->f_dentry, file, wait, ret, context);
}

void trace_ioctl_pre(struct trace_sb *sb, inode_t *inode, file_t *file, unsigned int cmd, unsigned long arg, void **context)
{
	MAKE_CALL4(sb, OP_IOCTL, ao_ioctl_pre, inode, file->f_dentry, inode, file, cmd, arg, context);
}

void trace_ioctl_post(struct trace_sb *sb, inode_t *inode, file_t *file, unsigned int cmd, unsigned long arg, int ret, void **context)
{
	MAKE_CALL5(sb, OP_IOCTL, ao_ioctl_post, inode, file->f_dentry, inode, file, cmd, arg, ret, context);
}
void trace_mmap_pre(struct trace_sb *sb, file_t *file, vm_area_t *vma, void **context)
{
	MAKE_CALL2(sb, OP_MMAP, ao_mmap_pre, file->f_dentry->d_inode, file->f_dentry, file, vma, context);
}

void trace_mmap_post(struct trace_sb *sb, file_t *file, vm_area_t *vma, int ret, void **context)
{
	MAKE_CALL3(sb, OP_MMAP, ao_mmap_post, file->f_dentry->d_inode, file->f_dentry, file, vma, ret, context);
}

void trace_flush_pre(struct trace_sb *sb, file_t *file, void **context)
{
	MAKE_CALL1(sb, OP_FLUSH, ao_flush_pre, file->f_dentry->d_inode, file->f_dentry, file, context);
}


void trace_flush_post(struct trace_sb *sb, file_t *file, int ret, void **context)
{
	MAKE_CALL2(sb, OP_FLUSH, ao_flush_post, file->f_dentry->d_inode, file->f_dentry, file, ret, context);
}

void trace_release_pre(struct trace_sb *sb, inode_t *inode, file_t *file, void **context)
{
	MAKE_CALL2(sb, OP_RELEASE, ao_release_pre, inode, file->f_dentry, inode, file, context);
}

void trace_release_post(struct trace_sb *sb, inode_t *inode, file_t *file, int ret, void **context)
{
	MAKE_CALL3(sb, OP_RELEASE, ao_release_post, inode, file->f_dentry, inode, file, ret, context);
}

void trace_fsync_pre(struct trace_sb *sb, file_t *file, dentry_t *dentry, int datasync, void **context)
{
	MAKE_CALL3(sb, OP_FSYNC, ao_fsync_pre, dentry->d_inode, dentry, file, dentry, datasync, context);
}

void trace_fsync_post(struct trace_sb *sb, file_t *file, dentry_t *dentry, int datasync, int ret, void **context)
{
	MAKE_CALL4(sb, OP_FSYNC, ao_fsync_post, dentry->d_inode, dentry, file, dentry, datasync, ret, context);
}

void trace_fasync_pre(struct trace_sb *sb, int fd, file_t *file, int flag, void **context)
{
	MAKE_CALL3(sb, OP_FASYNC, ao_fasync_pre, file->f_dentry->d_inode, file->f_dentry, fd, file, flag, context);
}

void trace_fasync_post(struct trace_sb *sb, int fd, file_t *file, int flag, int ret, void **context)
{
	MAKE_CALL4(sb, OP_FASYNC, ao_fasync_post, file->f_dentry->d_inode, file->f_dentry, fd, file, flag, ret, context);
}

void trace_lock_pre(struct trace_sb *sb, file_t *file, int cmd, struct file_lock *fl, void **context)
{
	MAKE_CALL3(sb, OP_LOCK, ao_lock_pre, file->f_dentry->d_inode, file->f_dentry, file, cmd, fl, context);
}

void trace_lock_post(struct trace_sb *sb, file_t *file, int cmd, struct file_lock *fl, int ret, void **context)
{
	MAKE_CALL4(sb, OP_LOCK, ao_lock_post, file->f_dentry->d_inode, file->f_dentry, file, cmd, fl, ret, context);
}
/* super ops */
void trace_read_inode_pre(struct trace_sb *sb, inode_t *inode, void **context)
{
	MAKE_CALL1(sb, OP_READ_INODE, ao_read_inode_pre, inode, NULL, inode, context);
}

void trace_read_inode_post(struct trace_sb *sb, inode_t *inode, void **context)
{
	MAKE_CALL1(sb, OP_READ_INODE, ao_read_inode_post, inode, NULL, inode, context);
}

void trace_write_inode_pre(struct trace_sb *sb, inode_t *inode, int sync, void **context)
{
	MAKE_CALL2(sb, OP_WRITE_INODE, ao_write_inode_pre, inode, NULL, inode, sync, context);
}

void trace_write_inode_post(struct trace_sb *sb, inode_t *inode, int sync, void **context)
{
	MAKE_CALL2(sb, OP_WRITE_INODE, ao_write_inode_post, inode, NULL, inode, sync, context);
}

void trace_put_inode_pre(struct trace_sb *sb, inode_t *inode, void **context)
{
	MAKE_CALL1(sb, OP_PUT_INODE, ao_put_inode_pre, inode, NULL, inode, context);
}

void trace_put_inode_post(struct trace_sb *sb, inode_t *inode, void **context)
{
	MAKE_CALL1(sb, OP_PUT_INODE, ao_put_inode_post, inode, NULL, inode, context);
}

void trace_delete_inode_pre(struct trace_sb *sb, inode_t *inode, void **context)
{
	MAKE_CALL1(sb, OP_DELETE_INODE, ao_delete_inode_pre, inode, NULL, inode, context);
}

void trace_delete_inode_post(struct trace_sb *sb, inode_t *inode, void **context)
{
	MAKE_CALL1(sb, OP_DELETE_INODE, ao_delete_inode_post, inode, NULL, inode, context);
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
void trace_statfs_pre(struct trace_sb *sb, struct statfs *buf, void **context)
#else
void trace_statfs_pre(struct trace_sb *sb, struct kstatfs *buf, void **context)
#endif
{
	MAKE_CALL1(sb, OP_STATFS, ao_statfs_pre, NULL, NULL, buf, context);
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
void trace_statfs_post(struct trace_sb *sb, struct statfs *buf, int ret, void **context)
#else
void trace_statfs_post(struct trace_sb *sb, struct kstatfs *buf, int ret, void **context)
#endif
{
	MAKE_CALL2(sb, OP_STATFS, ao_statfs_post, NULL, NULL, buf, ret, context);
}

void trace_clear_inode_pre(struct trace_sb *sb, inode_t *inode, void **context)
{
	MAKE_CALL1(sb, OP_CLEAR_INODE, ao_clear_inode_pre, inode, NULL, inode, context);
}

void trace_clear_inode_post(struct trace_sb *sb, inode_t *inode, void **context)
{
	MAKE_CALL1(sb, OP_CLEAR_INODE, ao_clear_inode_post, inode, NULL, inode, context);
}

/* dentry ops */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
void trace_d_revalidate_pre(struct trace_sb *sb, dentry_t *dentry, int flags, void **context)
{
	MAKE_CALL2(sb, OP_D_REVALIDATE, ao_d_revalidate_pre, dentry->d_inode, dentry, dentry, flags, context);
}
void trace_d_revalidate_post(struct trace_sb *sb, dentry_t *dentry, int flags, int ret, void **context)
{
	MAKE_CALL3(sb, OP_D_REVALIDATE, ao_d_revalidate_post, dentry->d_inode, dentry, dentry, flags, ret, context);
}
#else
void trace_d_revalidate_pre(struct trace_sb *sb, dentry_t *dentry, struct nameidata* nd, void **context)
{
  /*
   * XXX: tim: might not be correct, this is to handle the condition where a
   * NULL nameidata pointer is passed to tracefs
   */
  int flags = (nd) ? nd->flags : 0;
	MAKE_CALL2(sb, OP_D_REVALIDATE, ao_d_revalidate_pre, dentry->d_inode, dentry, dentry, flags, context);
}
void trace_d_revalidate_post(struct trace_sb *sb, dentry_t *dentry, struct nameidata* nd, int ret, void **context)
{
      /*
       * XXX: tim: might not be correct, this is to handle the condition where
       * a NULL nameidata pointer is passed to tracefs
       */
        int flags = (nd) ? nd->flags : 0;
	MAKE_CALL3(sb, OP_D_REVALIDATE, ao_d_revalidate_post, dentry->d_inode, dentry, dentry, flags, ret, context);
}
#endif

void trace_d_hash_pre(struct trace_sb *sb, dentry_t *dentry, qstr_t *name, void **context)
{
	MAKE_CALL2(sb, OP_D_HASH, ao_d_hash_pre, dentry->d_inode, dentry, dentry, name, context);
}

void trace_d_hash_post(struct trace_sb *sb, dentry_t *dentry, qstr_t *name, int ret, void **context)
{
	MAKE_CALL3(sb, OP_D_HASH, ao_d_hash_post, dentry->d_inode, dentry, dentry, name, ret, context);
}

void trace_d_compare_pre(struct trace_sb *sb, dentry_t *dentry, qstr_t *a, qstr_t *b, void **context)
{
	MAKE_CALL3(sb, OP_D_COMPARE, ao_d_compare_pre, dentry->d_inode, dentry, dentry, a, b, context);
}

void trace_d_compare_post(struct trace_sb *sb, dentry_t *dentry, qstr_t *a, qstr_t *b, int ret, void **context)
{
	MAKE_CALL4(sb, OP_D_COMPARE, ao_d_compare_post, dentry->d_inode, dentry, dentry, a, b, ret, context);
}

void trace_d_delete_pre(struct trace_sb *sb, dentry_t *dentry, void **context)
{
	MAKE_CALL1(sb, OP_D_DELETE, ao_d_delete_pre, dentry->d_inode, dentry, dentry, context);
}

void trace_d_delete_post(struct trace_sb *sb, dentry_t *dentry, int ret, void **context)
{
	MAKE_CALL2(sb, OP_D_DELETE, ao_d_delete_post, dentry->d_inode, dentry, dentry, ret, context);
}

void trace_d_release_pre(struct trace_sb *sb, dentry_t *dentry, void **context)
{
	MAKE_CALL1(sb, OP_D_RELEASE, ao_d_release_pre, dentry->d_inode, dentry, dentry, context);
}

void trace_d_release_post(struct trace_sb *sb, dentry_t *dentry, void **context)
{
	MAKE_CALL1(sb, OP_D_RELEASE, ao_d_release_post, dentry->d_inode, dentry, dentry, context);
}

void trace_d_iput_pre(struct trace_sb *sb, dentry_t *dentry, inode_t *inode, void **context)
{
	MAKE_CALL2(sb, OP_D_IPUT, ao_d_iput_pre, inode, dentry, dentry, inode, context);
}

void trace_d_iput_post(struct trace_sb *sb, dentry_t *dentry, inode_t *inode, void **context)
{
	MAKE_CALL2(sb, OP_D_IPUT, ao_d_iput_post, inode, dentry, dentry, inode, context);
}

void trace_readpage_pre(struct trace_sb *sb, struct file *file, struct page *page, void **context)
{
	MAKE_CALL2(sb, OP_READPAGE, ao_readpage_pre, file->f_dentry->d_inode, file->f_dentry, file, page, context);
}

void trace_readpage_post(struct trace_sb *sb, struct file *file, struct page *page, int ret, void **context)
{
	MAKE_CALL3(sb, OP_READPAGE, ao_readpage_post, file->f_dentry->d_inode, file->f_dentry, file, page, ret, context);
}

void trace_writepage_pre(struct trace_sb *sb, struct page *page, void **context)
{
	MAKE_CALL1(sb, OP_WRITEPAGE, ao_writepage_pre, page->mapping->host, NULL, page, context);
}

void trace_writepage_post(struct trace_sb *sb, struct page *page, int ret, void **context)
{
	MAKE_CALL2(sb, OP_WRITEPAGE, ao_writepage_post, page->mapping->host, NULL, page, ret, context);
}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
void trace_readpages_pre(struct trace_sb *sb, struct file *file, struct address_space* mapping, struct list_head *pages, void **context)
{
	MAKE_CALL3(sb, OP_READPAGES, ao_readpages_pre, file->f_dentry->d_inode, file->f_dentry, file, mapping, pages,context);
}

void trace_readpages_post(struct trace_sb *sb, struct file *file, struct address_space* mapping, struct list_head *pages, int ret, void **context)
{
	MAKE_CALL4(sb, OP_READPAGES, ao_readpages_post, file->f_dentry->d_inode, file->f_dentry, file, mapping, pages, ret, context);
}
#endif //#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)

void trace_prepare_write_pre(struct trace_sb *sb, struct file *file, struct page *page, unsigned to, unsigned from, void **context)
{
	MAKE_CALL4(sb, OP_PREPARE_WRITE, ao_prepare_write_pre, page->mapping->host, NULL, file, page, to, from, context);
}

void trace_prepare_write_post(struct trace_sb *sb, struct file *file, struct page *page, unsigned to, unsigned from, int ret, void **context)
{
	MAKE_CALL5(sb, OP_PREPARE_WRITE, ao_prepare_write_post, page->mapping->host, NULL, file, page, to, from, ret, context);
}

void trace_commit_write_pre(struct trace_sb *sb, struct file *file, struct page *page, unsigned to, unsigned from, void **context)
{
	MAKE_CALL4(sb, OP_COMMIT_WRITE, ao_commit_write_pre, page->mapping->host, NULL, file, page, to, from, context);
}

void trace_commit_write_post(struct trace_sb *sb, struct file *file, struct page *page, unsigned to, unsigned from, int ret, void **context)
{
	MAKE_CALL5(sb, OP_COMMIT_WRITE, ao_commit_write_post, page->mapping->host, NULL, file, page, to, from, ret, context);
}

void trace_sync_page_pre(struct trace_sb *sb,page_t* page,void** context)
{
    MAKE_CALL1(sb,OP_SYNC_PAGE,ao_sync_page_pre,page->mapping->host,NULL,page,context);
}

void trace_sync_page_post(struct trace_sb *sb,page_t* page,int ret,void** context)
{
    MAKE_CALL2(sb,OP_SYNC_PAGE,ao_sync_page_post,page->mapping->host,NULL,page,ret,context);
}

void trace_start(struct trace_sb *sb, int id)
{
    struct trace_destination *dest = &sb->dest[id];
    sb->dest[id].td_ad->ad_ops->ao_start_trace(sb->dest[id].td_ad, dest);
}

void trace_stop(struct trace_sb *sb, int id)
{
    struct trace_destination *dest = &sb->dest[id];
    sb->dest[id].td_ad->ad_ops->ao_stop_trace(sb->dest[id].td_ad, dest);
}

