/*
   Generic cipher template header

   $Id: gen-cipher.h,v 1.6 2006/11/28 01:12:21 ezk Exp $

   Written by Herbert Valerio Riedel <hvr@gnu.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software

   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "blowfish.h"

#if !defined(CIPHER_KEY_SIZE_MASK)
# error CIPHER_KEY_SIZE_MASK not defined
#endif

#if !defined(CIPHER_ID)
# error CIPHER_ID not defined
#endif

#if !defined(CIPHER_BLOCKSIZE)
#error CIPHER_BLOCKSIZE not defined
#endif

#if !defined(CIPHER_KEY_SCHEDULE_SIZE)
#error CIPHER_KEY_SCHEDULE_SIZE not defined
#endif

/* some cpp magic */
#define __STR(x) # x
#define __xSTR(x) __STR(x)

#define __CAT(x,y) x ## y
#define __xCAT(x,y) __CAT(x,y)

#define CIPHER_SYMBOL(x) __xCAT(CIPHER_ID,x)

#if !defined(CIPHER_STR)
# define CIPHER_STR __xSTR(CIPHER_ID)
#endif

/*
 * compat #defines for gen-{ecb,cbc,cfb,ctr,rtc}.h
 */

#define CIPHER_NAME(x) CIPHER_SYMBOL(x)

#if CIPHER_BLOCKSIZE == 64
# define CIPHER_BITS_64
#elif CIPHER_BLOCKSIZE == 128
# define CIPHER_BITS_128
#else
# error unsupported cipher blocksize
#endif


#define __xINIT_CIPHER_BLKOPS(name)    INIT_CIPHER_BLKOPS(name)
#define __xINIT_CIPHER_OPS(name)       INIT_CIPHER_OPS(name)

/* ECB */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
#include "gen-ecb.h"
static struct tf_cipher_implementation CIPHER_SYMBOL(_ecb) = {
  blocksize: CIPHER_BLOCKSIZE / 8,
  ivsize: 0,
  key_schedule_size: CIPHER_KEY_SCHEDULE_SIZE,
  key_size_mask: CIPHER_KEY_SIZE_MASK,
  __xINIT_CIPHER_BLKOPS (CIPHER_SYMBOL(_ecb)),
  __xINIT_CIPHER_OPS (CIPHER_ID)
};
#endif

/* CBC */

#include "gen-cbc.h"

static struct tf_cipher_implementation CIPHER_SYMBOL(_cbc) = {
  blocksize: CIPHER_BLOCKSIZE / 8,
  ivsize: CIPHER_BLOCKSIZE / 8,
  key_schedule_size: CIPHER_KEY_SCHEDULE_SIZE,
  key_size_mask: CIPHER_KEY_SIZE_MASK,
  __xINIT_CIPHER_BLKOPS (CIPHER_SYMBOL(_cbc)),
  __xINIT_CIPHER_OPS (CIPHER_ID)
};

/* CFB */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
#include "gen-cfb.h"
static struct tf_cipher_implementation CIPHER_SYMBOL(_cfb) = {
  blocksize: CIPHER_BLOCKSIZE / 8,
  ivsize: CIPHER_BLOCKSIZE / 8,
  key_schedule_size: CIPHER_KEY_SCHEDULE_SIZE,
  key_size_mask: CIPHER_KEY_SIZE_MASK,
  __xINIT_CIPHER_BLKOPS (CIPHER_SYMBOL(_cfb)),
  __xINIT_CIPHER_OPS (CIPHER_ID)
};
#endif

/* CTR */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
#include "gen-ctr.h"
static struct tf_cipher_implementation CIPHER_SYMBOL(_ctr) = {
  blocksize: CIPHER_BLOCKSIZE / 8,
  ivsize: CIPHER_BLOCKSIZE / 8,
  key_schedule_size: CIPHER_KEY_SCHEDULE_SIZE,
  key_size_mask: CIPHER_KEY_SIZE_MASK,
  __xINIT_CIPHER_BLKOPS (CIPHER_SYMBOL(_ctr)),
  __xINIT_CIPHER_OPS (CIPHER_ID)
};

/* RTC */

#include "gen-rtc.h"
static struct tf_cipher_implementation CIPHER_SYMBOL(_rtc) = {
  blocksize: CIPHER_BLOCKSIZE / 8,
  ivsize: CIPHER_BLOCKSIZE / 8,
  key_schedule_size: CIPHER_KEY_SCHEDULE_SIZE,
  key_size_mask: CIPHER_KEY_SIZE_MASK,
  __xINIT_CIPHER_BLKOPS (CIPHER_SYMBOL(_rtc)),
  __xINIT_CIPHER_OPS (CIPHER_ID)
};
#endif

/* EOF */
