/*
 * Copyright (c) 2003-2007 Erez Zadok
 * Copyright (c) 2003-2007 Stony Brook University
 * Copyright (c) 2003-2007 The Research Foundation of State University of New York
 * Copyright (c) 2004-2007 Nikolai Joukov
 * Copyright (c) 2003-2005 Akshat Aranya
 * Copyright (c) 2006-2007 Gopala Surya
 *
 * Dummy lib user to measure the interface overheads
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <assert.h>

#define NEED_DESCRIPTIVE_MESSAGES

#include "tracereader.h"
#include <unistd.h>
#include "util.h"

#include <linux/limits.h>
#include <linux/stat.h>
#include <linux/errno.h>

int main(int argc, char *argv[])
{
	int err;
	struct parse_params params;

	memset(&params, 0, sizeof(params));
	params.cparams.blk_size = 4096;

	if(argc != 3 || argv[1][0] != '-' || argv[1][1] != 'f') {
		fprintf(stderr, "Usage: dummy -f filename\n");
		return 1;
	}
	params.file_name = argv[2];
	params.ops[params.n_ops++] = OP_FILE;

	err = parse(&params);
	if (err != ERR_OK)
		fprintf(stderr, "Parsing failed with error code %d\n", err);
	return err;
}

void *on_start_trace_message(op_t type, int pre_or_post,void* cont)
{
	return (void*)0xACDC;
}

int on_end_trace_message(op_t type, int pre_or_post, void *cont)
{
	return 0;
}

void on_error(int error_type, int msg_num, char *desc)
{
	fprintf(stderr, "Error occurred while parsing trace: err_num = %d, msg_num = %d, description = %s\n", error_type, msg_num, desc);
}

void on_warning(int warn_type, char *desc)
{
	fprintf(stderr, "Warning: %s [%d]\n", desc, warn_type);
}

int on_data_field(op_t op, int pre_or_post, int tlv, field_t type, int length, void *value, void *cont)
{
	return ERR_OK;
}

int on_header(struct stream_header *header)
{
	return ERR_OK;
}

